@@ -90,10 +90,9 @@ file-like objects:
 
 __ver_major__ = 0
 __ver_minor__ = 3
-__ver_patch__ = 4
-__ver_sub__ = ""
-__version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,
-                              __ver_patch__,__ver_sub__)
+__ver_patch__ = 5
+__ver_sub__ = "p1"
+__version__ = "%d.%d.%d%s" % (__ver_major__,__ver_minor__,__ver_patch__,__ver_sub__)
 
 
 from StringIO import StringIO
