@@ -29,9 +29,6 @@ This module provides the dual filelike wrappers 'PadToBlockSize' and
 import filelike
 from filelike.wrappers import FileWrapper
 
-import unittest
-from StringIO import StringIO
-
 
 class PadToBlockSize(FileWrapper):
     """Class padding files to a fixed block size.
@@ -46,11 +43,16 @@ class PadToBlockSize(FileWrapper):
     """
 
     def __init__(self,fileobj,blocksize,mode=None):
-        super(PadToBlockSize,self).__init__(fileobj,mode)
         self.blocksize = blocksize
-        self._pad_loc = None
         self._pad_read = ""
         self._pad_unread = ""
+        super(PadToBlockSize,self).__init__(fileobj,mode)
+        if "a" in self.mode:
+            # Position at the start of the padding, since that's
+            # where any additional writes need to happen
+            self._pad_unread = self._pad_read
+            self._pad_read = ""
+            self._sbuffer = None
     
     def _round_up(self,num):
         """Round <num> up to a multiple of the block size."""
@@ -86,7 +88,6 @@ class PadToBlockSize(FileWrapper):
             sizehint = self._round_up(sizehint)
         data = self._fileobj.read(sizehint)
         if sizehint < 0 or len(data) < sizehint:
-            self._pad_loc = self._fileobj.tell()
             self._pad_unread = self._padding(data)
         return data
 
@@ -129,7 +130,12 @@ class PadToBlockSize(FileWrapper):
         # Remove the padding data from the leftovers
         if maybePad:
             zIdx = leftover.rfind("Z")
-            self._fileobj.write(leftover[:zIdx])
+            data = leftover[:zIdx]
+            padding = self._padding(data)
+            padstop = len(leftover) - zIdx
+            self._fileobj.write(data)
+            self._pad_read = padding[:padstop]
+            self._pad_unread = padding[padstop:]
         return None
 
     def _seek(self,offset,whence):
@@ -140,26 +146,40 @@ class PadToBlockSize(FileWrapper):
         the file and its padding, you'll be placed at EOF.
         """
         if whence > 0:
+            # TODO: implementing these shouldn't be that hard...
             raise NotImplementedError
-        # Slow simulation of seek by actually re-reading all the data.
+        print "SEEK:", offset, whence
         self._fileobj.seek(0,0)
         self._pad_unread = ""
         self._pad_read = ""
-        data = self._fileobj.read(offset)
-        boundary = self._round_down(len(data))
-        # 1) We're within the file, so it's nice and easy
-        if len(data) == offset:
-            self._fileobj.seek(boundary-offset,1)
-            return data[boundary:offset]
-        # 2) We're past the underlying file.
-        padding = self._padding(data)
-        self._fileobj.seek(boundary-len(data),1)
-        diff = offset - len(data)
-        # If we're inside the padding, only return what's necessary.
-        # Otherwise, we've seeked to the end of the whole thing.
-        if diff <= len(padding):
-            padding = padding[:diff]
-        return data[boundary:] + padding
+        if offset == 0:
+            return None
+        # Slow simulation of seek by actually re-reading all the data.
+        boundary = self._round_down(offset)
+        bytes_read = 0
+        while bytes_read < boundary:
+            data = self._fileobj.read(min(self._bufsize,boundary-bytes_read))
+            if data == "":
+                break
+            bytes_read += len(data)
+        # If the boundary is not within the file, we must have seeked right
+        # to (or past) the end of the padding.  So just position at end.
+        if bytes_read < boundary:
+            data = ""
+            self._pad_read = self._padding("A"*(bytes_read % self.blocksize))
+            return None
+        # Otherwise, we may have to return some data from the underlying file
+        pos = self._fileobj.tell()
+        data = self._fileobj.read(offset-boundary)
+        self._fileobj.seek(-1*len(data),1)
+        assert self._fileobj.tell() == pos, "peeking failed"
+        diff = offset - (len(data) + bytes_read)
+        assert diff >= 0, "peeking failed"
+        if diff > 0:
+            # The target offset is somewhere in the padding
+            padding = self._padding(data)
+            return data + padding[:diff]
+        return data
 
     def _tell(self):
         return self._fileobj.tell() + len(self._pad_read)
@@ -172,11 +192,13 @@ class UnPadToBlockSize(FileWrapper):
     removing extraneous padding data when reading, and adding it back in
     when writing.
     """
+
+    _append_requires_overwrite = True
     
     def __init__(self,fileobj,blocksize,mode=None):
-        super(UnPadToBlockSize,self).__init__(fileobj,mode)
         self.blocksize = blocksize
         self._pad_seen = ""
+        super(UnPadToBlockSize,self).__init__(fileobj,mode)
 
     def _round_up(self,num):
         """Round <num> up to a multiple of the block size."""
@@ -232,7 +254,11 @@ class UnPadToBlockSize(FileWrapper):
 
     def _write(self,data,flushing=False):
         """Write the given string to the file."""
-        size = self._round_down(len(data))
+        #  To ensure the pad is written, we always return non-empty leftovers.
+        #  This forces a flushing write on file close.
+        if data == "":
+            return ""
+        size = self._round_down(len(data)-1)
         self._fileobj.write(data[:size])
         leftover = data[size:]
         if not flushing:
@@ -240,7 +266,7 @@ class UnPadToBlockSize(FileWrapper):
         # Flushing, so we need to pad the data.  If the file is readable,
         # check to see if we're in the middle and pad using existing data.
         if self._check_mode("r"):
-            lenNB = self._round_up(len(leftover))
+            lenNB = self._round_up(len(leftover)+1)
             nextBlock = self._fileobj.read(lenNB)
             self._fileobj.seek(-1*len(nextBlock),1)
             if lenNB == len(nextBlock):
@@ -259,11 +285,13 @@ class UnPadToBlockSize(FileWrapper):
             raise NotImplementedError
         self._fileobj.seek(0,0)
         self._pad_seen = ""
+        if offset == 0:
+            return None
         data = self._fileobj.read(offset)
         eof = data.rfind("Z")
         if len(data) < offset:
             offset = eof
-        elif eof > len(data) - self.blocksize - 1:
+        elif eof != -1 and eof > len(data) - self.blocksize - 1:
             extra = self._fileobj.read(self.blocksize+1)
             data = data + extra
             if len(extra) <= self.blocksize:
@@ -277,125 +305,3 @@ class UnPadToBlockSize(FileWrapper):
     def _tell(self):
         return self._fileobj.tell() - len(self._pad_seen)
 
-
-class Test_PadToBlockSize5(filelike.Test_ReadWriteSeek):
-    """Testcases for PadToBlockSize with blocksize=5."""
-
-    contents = "this is some sample textZ"
-    text_plain = ["Zhis is sample texty"]
-    text_padded = ["Zhis is sample textyZXXXX"]
-    blocksize = 5
-
-    def makeFile(self,contents,mode):
-        # Careful here - 'contents' should be the contents of the returned
-        # file, and is therefore expected to contain the padding.  But for
-        # easy testing we allow it to omit the padding and be used directly
-        # in the underlying StringIO object.
-        idx = contents.rfind("Z")
-        if idx < 0:
-            idx = len(contents)
-        f = PadToBlockSize(StringIO(contents[:idx]),self.blocksize,mode=mode)
-        return f
-
-    def test_padding(self):
-        for (plain,padded) in zip(self.text_plain,self.text_padded):
-            f = self.makeFile(padded,"rw")
-            self.assert_(len(padded) % self.blocksize == 0)
-            self.assertEquals(f._fileobj.getvalue(),plain)
-
-    def test_read_empty_file(self):
-        # The empty file should still yield padding
-        f = self.makeFile("","r")
-        self.assertEquals(f.read(),"Z"+"X"*(f.blocksize-1))
-
-    def test_write_zeds(self):
-        f = self.makeFile("","w")
-        txt = "test data Z with lots of Z's embedded in it Z"
-        f.write("test data Z w")
-        f.write("ith lots of Z's e")
-        f.write("mbedded in it Z")
-        f.write(f._padding(txt))
-        f.flush()
-        self.assertEquals(f._fileobj.getvalue(),txt)
-
-
-class Test_PadToBlockSize7(Test_PadToBlockSize5):
-    """Testcases for PadToBlockSize with blocksize=7."""
-
-    contents = "this is som\n sample textZXXX"
-    text_plain = ["Zhis is sample texty"]
-    text_padded = ["Zhis is sample textyZ"]
-    blocksize = 7
-
-
-class Test_PadToBlockSize16(Test_PadToBlockSize5):
-    """Testcases for PadToBlockSize with blocksize=16."""
-
-    contents = "This is Zome Zample TeZTZXXXXXXX"
-    text_plain = ["short"]
-    text_padded = ["shortZXXXXXXXXXX"]
-    blocksize = 16
-
-
-class Test_UnPadToBlockSize5(filelike.Test_ReadWriteSeek):
-    """Testcases for UnPadToBlockSize with blocksize=5."""
-
-    contents = "this is some sample text"
-    text_plain = ["Zhis is sample texty"]
-    text_padded = ["Zhis is sample textyZXXXX"]
-    blocksize = 5
-
-    def makeFile(self,contents,mode):
-        f = UnPadToBlockSize(StringIO(""),self.blocksize,mode=mode)
-        f._fileobj = StringIO(contents + f._padding(contents))
-        return f
-
-    def test_padding(self):
-        for (plain,padded) in zip(self.text_plain,self.text_padded):
-            f = self.makeFile(plain,"rw")
-            self.assertEquals(f._fileobj.getvalue(),padded)
-
-    def test_write_zeds(self):
-        f = self.makeFile("","w")
-        txt = "test data Z with lots of Z's embedded in it Z"
-        f.write("test data Z w")
-        f.write("ith lots of Z's e")
-        f.write("mbedded in it Z")
-        f.flush()
-        self.assertEquals(f._fileobj.getvalue(),txt + f._padding(txt))
-
-    def test_read_zeds(self):
-        f = self.makeFile("","r")
-        txt = "test data Z with lots of Z's embedded in it Z"
-        f._fileobj = StringIO(txt + f._padding(txt))
-        self.assertEquals(f.read(),txt)
-
-
-class Test_UnPadToBlockSize7(Test_UnPadToBlockSize5):
-    """Testcases for UnPadToBlockSize with blocksize=7."""
-
-    contents = "this is som\n sample text"
-    text_plain = ["Zhis is sample texty"]
-    text_padded = ["Zhis is sample textyZ"]
-    blocksize = 7
-
-
-class Test_UnPadToBlockSize16(Test_UnPadToBlockSize5):
-    """Testcases for UnPadToBlockSize with blocksize=16."""
-
-    contents = "This is Zome Zample TeZTZ"
-    text_plain = ["short"]
-    text_padded = ["shortZXXXXXXXXXX"]
-    blocksize = 16
-
-
-def testsuite():
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(Test_PadToBlockSize5))
-    suite.addTest(unittest.makeSuite(Test_PadToBlockSize7))
-    suite.addTest(unittest.makeSuite(Test_PadToBlockSize16))
-    suite.addTest(unittest.makeSuite(Test_UnPadToBlockSize5))
-    suite.addTest(unittest.makeSuite(Test_UnPadToBlockSize7))
-    suite.addTest(unittest.makeSuite(Test_UnPadToBlockSize16))
-    return suite
-
